*** old-draw.c	Thu Sep 27 20:26:36 2001
--- draw.c	Thu Sep 27 23:18:38 2001
***************
*** 34,39 ****
--- 34,42 ----
  static int draw_fillellipse(SDL_Surface *dst, int x, int y, int rx, int ry, Uint32 color);
  static int draw_fillpoly(SDL_Surface *dst, int *vx, int *vy, int n, Uint32 color);
  
+ static void  drawvertline(SDL_Surface* surf, Uint32 color, int startx, int starty, int endy);
+ static int set_at(SDL_Surface* surf, int x, int y, Uint32 color);
+ 
  
  
      /*DOC*/ static char doc_line[] =
***************
*** 236,242 ****
      /*DOC*/    "pygame.draw.ellipse(Surface, color, Rect, width) -> Rect\n"
      /*DOC*/    "draw an ellipse on a surface\n"
      /*DOC*/    "\n"
!     /*DOC*/    "Draws a circular shape on the Surface. The given rectangle\n"
      /*DOC*/    "is the area that the circle will fill. The width argument is\n"
      /*DOC*/    "the thickness to draw the outer edge. If width is zero then\n"
      /*DOC*/    "the ellipse will be filled.\n"
--- 239,245 ----
      /*DOC*/    "pygame.draw.ellipse(Surface, color, Rect, width) -> Rect\n"
      /*DOC*/    "draw an ellipse on a surface\n"
      /*DOC*/    "\n"
!     /*DOC*/    "Draws an elliptical shape on the Surface. The given rectangle\n"
      /*DOC*/    "is the area that the circle will fill. The width argument is\n"
      /*DOC*/    "the thickness to draw the outer edge. If width is zero then\n"
      /*DOC*/    "the ellipse will be filled.\n"
***************
*** 338,343 ****
--- 341,352 ----
  	else
  		return RAISE(PyExc_TypeError, "invalid color argument");
  
+     if ( radius < 0 )
+ 		return RAISE(PyExc_ValueError, "negative radius");
+     if ( width < 0 )
+ 		return RAISE(PyExc_ValueError, "negative width");
+     if ( width > radius )
+ 		return RAISE(PyExc_ValueError, "width greater than radius");
  
  	if(!PySurface_Lock(surfobj)) return NULL;
  
***************
*** 371,377 ****
      /*DOC*/    "Draws a polygonal shape on the Surface. The given pointlist\n"
      /*DOC*/    "is the vertices of the polygon. The width argument is\n"
      /*DOC*/    "the thickness to draw the outer edge. If width is zero then\n"
!     /*DOC*/    "the ellipse will be filled.\n"
      /*DOC*/    "\n"
      /*DOC*/    "The color argument can be either a RGB sequence or mapped color integer.\n"
      /*DOC*/    "\n"
--- 380,386 ----
      /*DOC*/    "Draws a polygonal shape on the Surface. The given pointlist\n"
      /*DOC*/    "is the vertices of the polygon. The width argument is\n"
      /*DOC*/    "the thickness to draw the outer edge. If width is zero then\n"
!     /*DOC*/    "the polygonal shape will be filled.\n"
      /*DOC*/    "\n"
      /*DOC*/    "The color argument can be either a RGB sequence or mapped color integer.\n"
      /*DOC*/    "\n"
***************
*** 474,485 ****
  
      /*DOC*/ static char doc_rect[] =
      /*DOC*/    "pygame.draw.rect(Surface, color, Rect, width) -> Rect\n"
!     /*DOC*/    "draws a polygon on a surface\n"
      /*DOC*/    "\n"
!     /*DOC*/    "Draws a polygonal shape on the Surface. The given Rect\n"
      /*DOC*/    "is the area of the rectangle. The width argument is\n"
      /*DOC*/    "the thickness to draw the outer edge. If width is zero then\n"
!     /*DOC*/    "the ellipse will be filled.\n"
      /*DOC*/    "\n"
      /*DOC*/    "The color argument can be either a RGB sequence or mapped color integer.\n"
      /*DOC*/    "\n"
--- 483,494 ----
  
      /*DOC*/ static char doc_rect[] =
      /*DOC*/    "pygame.draw.rect(Surface, color, Rect, width) -> Rect\n"
!     /*DOC*/    "draws a rectangle on a surface\n"
      /*DOC*/    "\n"
!     /*DOC*/    "Draws a rectangle on the Surface. The given Rect\n"
      /*DOC*/    "is the area of the rectangle. The width argument is\n"
      /*DOC*/    "the thickness to draw the outer edge. If width is zero then\n"
!     /*DOC*/    "the rectangle will be filled.\n"
      /*DOC*/    "\n"
      /*DOC*/    "The color argument can be either a RGB sequence or mapped color integer.\n"
      /*DOC*/    "\n"
***************
*** 531,536 ****
--- 540,548 ----
  	if(pts[1] == pts[3])
  		drawhorzline(surf, color, pts[0], pts[1], pts[2]);
  	else
+ 	if(pts[0] == pts[2])
+ 		drawvertline(surf, color, pts[0], pts[1], pts[3]);
+ 	else
  		drawline(surf, color, pts[0], pts[1], pts[2], pts[3]);
  	return 1;
  }
***************
*** 780,797 ****
  	}
  }
  
  static void drawhorzlineclip(SDL_Surface* surf, Uint32 color, int x1, int y1, int x2)
  {
! 	if(y1 < surf->clip_rect.y || y1 > surf->clip_rect.y + surf->clip_rect.h)
  		return;
  	x1 = max(x1, surf->clip_rect.x);
! 	x2 = min(x2, surf->clip_rect.x + surf->clip_rect.w);
! 	if(x2 - x1 < 1)
  		return;
  	drawhorzline(surf, color, x1, y1, x2);
  }
  
  
  static int set_at(SDL_Surface* surf, int x, int y, Uint32 color)
  {
  	SDL_PixelFormat* format = surf->format;
--- 792,882 ----
  	}
  }
  
+ 
  static void drawhorzlineclip(SDL_Surface* surf, Uint32 color, int x1, int y1, int x2)
  {
!     int     temp;
! 
! 	if(y1 < surf->clip_rect.y || y1 >= surf->clip_rect.y + surf->clip_rect.h)
  		return;
+     if ( x2 < x1 ) {
+         temp = x1; x1 = x2; x2 = temp;
+     }
  	x1 = max(x1, surf->clip_rect.x);
! 	x2 = min(x2, surf->clip_rect.x + surf->clip_rect.w-1);
! 	if(x1 == x2) {
!         set_at( surf, x1, y1, color);
  		return;
+ 	}
  	drawhorzline(surf, color, x1, y1, x2);
  }
  
  
+ static void drawvertline(SDL_Surface* surf, Uint32 color, int x1, int y1, int y2)
+ {
+ 	Uint8   *pixel, *end;
+ 	Uint8   *colorptr;
+ 	Uint32  pitch = surf->pitch;
+ 
+ 	if(y1 == y2) return;
+ 
+     pixel = ((Uint8*)surf->pixels) + x1 * surf->format->BytesPerPixel;
+ 	if(y1 < y2)
+ 	{
+ 	    end    = pixel + surf->pitch * y2;
+ 	    pixel += surf->pitch * y1;
+ 	}
+ 	else
+ 	{
+ 	    end    = pixel + surf->pitch * y1;
+ 	    pixel += surf->pitch * y2;
+ 	}
+ 
+ 	switch(surf->format->BytesPerPixel)
+ 	{
+ 	case 1:
+ 		for(; pixel <= end; pixel+=pitch) {
+ 			*pixel = (Uint8)color;
+ 		}break;
+ 	case 2:
+ 		for(; pixel <= end; pixel+=pitch) {
+ 			*(Uint16*)pixel = (Uint16)color;
+ 		}break;
+ 	case 3:
+ 		if(SDL_BYTEORDER == SDL_BIG_ENDIAN) color <<= 8;
+ 		colorptr = (Uint8*)&color;
+ 		for(; pixel <= end; pixel+=pitch) {
+ 			pixel[0] = colorptr[0];
+ 			pixel[1] = colorptr[1];
+ 			pixel[2] = colorptr[2];
+ 		}break;
+ 	default: /*case 4*/
+ 		for(; pixel <= end; pixel+=pitch) {
+ 			*(Uint32*)pixel = color;
+ 		}break;
+ 	}
+ }
+ 
+ 
+ static void drawvertlineclip(SDL_Surface* surf, Uint32 color, int x1, int y1, int y2)
+ {
+     int     temp;
+ 
+     if(x1 < surf->clip_rect.x || x1 >= surf->clip_rect.x + surf->clip_rect.w)
+         return;
+     if ( y2 < y1 ) {
+         temp = y1; y1 = y2; y2 = temp;
+     }
+ 	y1 = max(y1, surf->clip_rect.y);
+ 	y2 = min(y2, surf->clip_rect.y + surf->clip_rect.h-1);
+ 	if(y2 - y1 < 1) {
+         set_at( surf, x1, y1, color);
+ 		return;
+     }
+ 	drawvertline(surf, color, x1, y1, y2);
+ }
+ 
+ 
  static int set_at(SDL_Surface* surf, int x, int y, Uint32 color)
  {
  	SDL_PixelFormat* format = surf->format;
***************
*** 836,849 ****
  	int xmj, xpj, ymi, ypi;
  	int xmk, xpk, ymh, yph;
  	
! #if 0
  	/* Special case for rx=0 - draw a vline */
! 	if (rx==0)
! 		return(vlineColor (dst, x, (Sint16)(y-ry), (Sint16)(y+ry), color));  
  	/* Special case for ry=0 - draw a hline */
! 	if (ry==0)
! 		return(hlineColor (dst, (Sint16)(x-rx), (Sint16)(x+rx), y, color));  
  	
  	/* Test bounding box */
  	x1=x-rx;
  	y1=y-ry;
--- 921,942 ----
  	int xmj, xpj, ymi, ypi;
  	int xmk, xpk, ymh, yph;
  	
! 	/* Special case - draw a single pixel */
! 	if (rx==0 && ry==0) {
!         return set_at( dst, x, y, color);
! 	}
  	/* Special case for rx=0 - draw a vline */
! 	if (rx==0) {
! 		drawvertlineclip( dst, color, x, (Sint16)(y-ry), (Sint16)(y+ry) );
! 		return 1;
! 	}
  	/* Special case for ry=0 - draw a hline */
! 	if (ry==0) {
! 		drawhorzlineclip( dst, color, (Sint16)(x-rx), y, (Sint16)(x+rx) );
! 		return 1;
! 	}
  	
+ #if 0
  	/* Test bounding box */
  	x1=x-rx;
  	y1=y-ry;
***************
*** 962,983 ****
  	int ix, iy;
  	int h, i, j, k;
  	int oh, oi, oj, ok;
! 	int xmh, xph;
! 	int xmi, xpi;
! 	int xmj, xpj;
! 	int xmk, xpk;
! 	
  	
! #if 0
  	/* Special case for rx=0 - draw a vline */
  	if (rx==0) {
! 		return(vlineColor (dst, x, (Sint16)(y-ry), (Sint16)(y+ry), color));  
  	}
  	/* Special case for ry=0 - draw a hline */
  	if (ry==0) {
! 		return(hlineColor (dst, (Sint16)(x-rx), (Sint16)(x+rx), y, color));  
  	}
  	
  	/* Test bounding box */
  	x1=x-rx;
  	y1=y-ry;
--- 1055,1081 ----
  	int ix, iy;
  	int h, i, j, k;
  	int oh, oi, oj, ok;
! 	int xmh, xph, ypk, ymk;
! 	int xmi, xpi, ymj, ypj;
! 	int xmj, xpj, ymi, ypi;
! 	int xmk, xpk, ymh, yph;
  	
! 	/* Special case - draw a single pixel */
! 	if (rx==0 && ry==0) {
!         return set_at( dst, x, y, color);
! 	}
  	/* Special case for rx=0 - draw a vline */
  	if (rx==0) {
! 		drawvertlineclip( dst, color, x, (Sint16)(y-ry), (Sint16)(y+ry) );
! 		return 1;
  	}
  	/* Special case for ry=0 - draw a hline */
  	if (ry==0) {
! 		drawhorzlineclip( dst, color, (Sint16)(x-rx), y, (Sint16)(x+rx) );
! 		return 1;
  	}
  	
+ #if 0
  	/* Test bounding box */
  	x1=x-rx;
  	y1=y-ry;
***************
*** 996,1031 ****
  	if (rx > ry) {
  		ix = 0;
  		iy = rx * 64;
- 		
  		do {
  			h = (ix + 32) >> 6;
  			i = (iy + 32) >> 6;
  			j = (h * ry) / rx;
  			k = (i * ry) / rx;
  			
! 			if ((ok!=k) && (oj!=k)) {
  				xph=x+h;
  				xmh=x-h;
  				if (k>0) {
! 					drawhorzlineclip(dst, color, xmh, y+k, xph);
! 					drawhorzlineclip(dst, color, xmh, y-k, xph);
  				} else {
  					drawhorzlineclip(dst, color, xmh,  y, xph);
  				}
  				ok=k;
- 			}
- 			if ((oj!=j) && (ok!=j) && (k!=j))  {
- 				xmi=x-i;
  				xpi=x+i;
  				if (j>0) {
! 					drawhorzlineclip(dst, color, xmi, y+j, xpi);
! 					drawhorzlineclip(dst, color, xmi, y-j, xpi);
  				} else {
  					drawhorzlineclip(dst, color, xmi, y, xpi);
  				}
  				oj=j;
  			}
- 			
  			ix = ix + iy / rx;
  			iy = iy - ix / rx;
  			
--- 1095,1130 ----
  	if (rx > ry) {
  		ix = 0;
  		iy = rx * 64;
  		do {
  			h = (ix + 32) >> 6;
  			i = (iy + 32) >> 6;
  			j = (h * ry) / rx;
  			k = (i * ry) / rx;
  			
! 			if (((ok!=k) && (oj!=k)) || ((oj!=j) && (ok!=j)) || (k!=j)) {
  				xph=x+h;
  				xmh=x-h;
  				if (k>0) {
! 					ypk=y+k;
! 					ymk=y-k;
! 		            drawhorzlineclip( dst, color, xmh, ypk, xph );
! 		            drawhorzlineclip( dst, color, xmh, ymk, xph );
  				} else {
  		            drawhorzlineclip( dst, color, xmh, y, xph );
  				}
  				ok=k;
  				xpi=x+i;
+ 				xmi=x-i;
  				if (j>0) {
! 					ypj=y+j;
! 					ymj=y-j;
! 		            drawhorzlineclip( dst, color, xmi, ypj, xpi );
! 		            drawhorzlineclip( dst, color, xmi, ymj, xpi );
  				} else {
  		            drawhorzlineclip( dst, color, xmi, y, xpi );
  				}
  				oj=j;
  			}
  			ix = ix + iy / rx;
  			iy = iy - ix / rx;
  			
***************
*** 1033,1076 ****
  	} else {
  		ix = 0;
  		iy = ry * 64;
- 		
  		do {
  			h = (ix + 32) >> 6;
  			i = (iy + 32) >> 6;
  			j = (h * rx) / ry;
  			k = (i * rx) / ry;
  			
! 			if ((oi!=i) && (oh!=i)) {
  				xmj=x-j;
  				xpj=x+j; 
  				if (i>0) {
! 					drawhorzlineclip(dst, color, xmj, y+i, xpj);
! 					drawhorzlineclip(dst, color, xmj, y-i, xpj);
  				} else {
  					drawhorzlineclip(dst, color, xmj, y, xpj);
  				}
  				oi=i;
- 			}
- 			if ((oh!=h) && (oi!=h) && (i!=h)) {
  				xmk=x-k;
  				xpk=x+k;
  				if (h>0) {
! 					drawhorzlineclip(dst, color, xmk, y+h, xpk);
! 					drawhorzlineclip(dst, color, xmk, y-h, xpk);
  				} else {
  					drawhorzlineclip(dst, color, xmk, y, xpk);
  				}
  				oh=h;
  			}
- 			
  			ix = ix + iy / ry;
  			iy = iy - ix / ry;
- 			
  		} while(i > h);
  	}
  	
! 	return(result);
  }
  
  
  static int compare_int(const void *a, const void *b)
--- 1132,1176 ----
  	} else {
  		ix = 0;
  		iy = ry * 64;
  		do {
  			h = (ix + 32) >> 6;
  			i = (iy + 32) >> 6;
  			j = (h * rx) / ry;
  			k = (i * rx) / ry;
  			
! 			if (((oi!=i) && (oh!=i)) || ((oh!=h) && (oi!=h) && (i!=h))) {
  				xmj=x-j;
  				xpj=x+j;
  				if (i>0) {
! 					ypi=y+i;
! 					ymi=y-i;
! 		            drawhorzlineclip( dst, color, xmj, ypi, xpj );
! 		            drawhorzlineclip( dst, color, xmj, ymi, xpj );
  				} else {
  		            drawhorzlineclip( dst, color, xmj, y, xpj );
  				}
  				oi=i;
  				xmk=x-k;
  				xpk=x+k;
  				if (h>0) {
! 					yph=y+h;
! 					ymh=y-h;
! 		            drawhorzlineclip( dst, color, xmk, yph, xpk );
! 		            drawhorzlineclip( dst, color, xmk, ymh, xpk );
  				} else {
  		            drawhorzlineclip( dst, color, xmk, y, xpk );
  				}
  				oh=h;
  			}
  			ix = ix + iy / ry;
  			iy = iy - ix / ry;
  		} while(i > h);
  	}
  	
! 	return 1;
  }
+ 
+ 
  
  
  static int compare_int(const void *a, const void *b)
